**************************************************************************************
*By Using This Library To Agree To Have Read, Understood And Agreed To The Following:*
**************************************************************************************

This library and the information contained therein is free to use
ONLY if given consent by the author ("AJX")

Addendum to the above: You may use this library without direct consent
from the author if you are using it for personal & non-profit purposes.
Commercial or any for-profit uses, direct or indirect, require permission
from AJX.

Any content explicitly labeled to be property of someone other than
AJX is copyrighted by its author and all rights are reserved unless
otherwise stipulated by the respective owner(s).

All other content is owned and copyrighted by the author, AJX.
All rights are reserved.

If you have been given permission to use this library please give credit
where credit is due, not just to the author but the authors of all
included content. (Theodis, /*Lummox Jr.,*/ and others)

This library is provided "as-is" with no guarantees or warranties of any kind.

Note: The code provided by Lummox Jr was removed
**************************************************************************************
**************************************************************************************
**************************************************************************************

**************************************
*	Table of Contents	             *
**************************************
1: Explanation
	a. Purpose
	b. Summary
	c. Written Demonstration
2: Procs and Their Uses
	a. Enter()
	b. Entered()
	c. Exit()
	d. Exited()
	e. Fall()
	f. LandedOn()
	g. Landed()
	h. ShadowUpdate()




**************************************
*	Explanation			             *
**************************************

This library is intended to allow full range 3D motion in an isometric map.

The design behind this setup is very simple.
Anything that is dense can be collided with, both in normal flat ground
motion as well as jumping on. This means you can jump on objects that sit
on the ground, as well as turfs or other players.

The library includes existing procs incorporated in similar manners, as
well as a few new procs to handle situations that would not have been
possible in a 2D environment.


Generally in a BYOND environment movement is determined by density.
If a turf is dense, you cannot enter it if you are also dense.
If a turf has an object inside of it that is dense, you cannot enter if you are also dense.

In this system density is more 'true' to its real life definition.
If something is dense, it can be collided with.
If something is not dense, you will never physically interact with it.
(Meaning no bumps() or stepping on it)

So, if you try to stand on a floating platform that is not dense, you will simply
fall through it.

The order of operations as you try to enter a turf works like this:

1:	 atom/movable/Move(Loc,Dir)
Move is called. Generally speaking 'Loc' isn't relevant here.
Unlike traditional Move(), dir is actually the direction you're moving in,
not the direction you will be facing when you get there.

2:	if(get_dist(src,Loc)>1)
***Edit: This line has been removed. ***
//This system isnt made to handle distances greater than 1
//So for now it simply forces movement to Loc

3: 	var/atom/Destination=GetStepD(src,Dir)
GetStep() tells you what you would land on if you were to step from
loc to get_step(src,Dir). GetStepD(src,NewLoc) Tells you where you would
land if you were to step from where you are to NewLoc.
It takes many things into account such as the Height of the person moving
as well as how high up they can step. For this reason if you are trying to
use GetStep() for pathfinding you may need to have a dummy unit. I will
create a new proc for doing remote checks rather than from a specific source,
but for the first release that isn't available.


3b: If there is no destination returned by GetStep() that means there are no
valid moves, so the attempted move fails.

3c: If GetStep() returns a list that means that the person moving bumped something,
so it calls Bump() (which in turn calls Bumped()) and fails the move.

4: If the moving atom doesn't have a Floor set, then it assumes you're standing on
your turf

5: Checks Floor.Exit(). If this returns 0, the move fails
6: Checks Destination.Enter(). If this returns 0, the move fails

7: If the above procs returned non-zero values then you *should* be able to move,
so it calls Floor.Exited()

8: If your current Floor is not a turf then it removes you from its Contents.
Note the capitalized C. All atoms have a Contents list that is auto initialized/deleted
when in use/not in use respectively. The purpose here is to check to see if a person
is standing 'on' something. This doesn't only include people who are standing directly
on the object in question, but also those who are in the air directly above that object.

9: Your new Floor = Your destination atom

10: If your new Floor is a turf, you can simply set your loc there and the turfs built in
contents will update itself. If not, move the moving atom to the new Floor's loc,
create the Contents if necessary then add the mover to it.

11: Call Entered() for the new Floor

12: Call Fall(), which is the gravity handling proc. If you ever have a tile that
moves or opens you will need to call Fall() manually. This proc is not left running
constantly to keep overhead down.

13: Call ShadowUpdate(). This proc will update the shadow for the object in question.
Shadows help add a great deal of perspective when viewing the map, and make understanding
where things are considerably easier. As such this should always be used,
even potentially with unique shadows if possible.



An example of the result this system creates is this:

Stepping from source (0 pixel_z) to destination (0 pixel_z) will work
as long as there is nothing on the destination tile.

Stepping from source (10 pixel_z) to destination (0 pixel_z) will also work.
When you move you will retain your pixel_z, but you will begin to fall according
to the gravity speed until you land on the destination.

Stepping from source (0 pixel_z) to destination (4 pixel_z) will work with default settings,
because the highest you can step-up to is 4 pixels. Your height will immediately be updated
to the destination pixel_z.

Stepping from source (10 pixel_z) to destination (0 pixel_z) with a rock (4 height) will work.
You will land on the rock, with a 4 pixel_z.

Stepping from source (0 pixel_z) to destination (0 pixel_z) that has a platform floating at
8 pixel_z will not work, because it is too low to walk under with 10 height (default mob height)

You cannot 'step up' while falling, you must be equal to or higher than the pixel_z of your
destination.

You cannot 'step up' to somewhere you will hit your head on




**************************************
*	Procs and Their Uses             *
**************************************
	a. atom/Enter(atom/movable/A)

Called when A is attempting to enter src. A may be either standing
on top of src or in the air above src, as long as src is the nearest thing
to land on top of.

Returns 0 to disallow movement, non-zero to allow.



	b. atom/Entered(atom/movable/M,atom/OldLoc)

Called after M has successfully moved into src. OldLoc is M's previous location.



	c. atom/Exit(atom/movable/A)

Called when A is attempting to leave src. Again, doesn't need to
be standing directly on src; A can also be in the air directly
above src.

Returns 0 to disallow movement, non-zero to allow.



	d. atom/Exited(atom/movable/M)

Called after M has successfully left src.



	e. atom/proc/Fall()

Called after src has moved to a new location. If src's pixel_z
is higher than src.Floor.pixel_z+Floor.Height then it moves src up.
If src.Floor.pixel_z+Floor.Height is lower then src begins to fall
until it lands on Floor.

Once src is actually touching Floor it calls src.Landed() and Floor.LandedOn()

	f. atom/proc/LandedOn(atom/movable/A)

Called when A lands on src.



	g. atom/proc/Landed(atom/A)

Called when src lands on A



	h. atom/movable/proc/ShadowUpdate()

Called after Move()ing. Should be written to update your Shadow.icon_state.
If you feel like being detailed you could theoretically make separate Shadows
for each and every type of object. For the sake of the demo I merely used one.